package com.biganto.visual.roompark.domain.use_case

import com.biganto.visual.roompark.data.data_provider.featuresVariants
import com.biganto.visual.roompark.data.repository.db.requrey.model.EstateEntity
import com.biganto.visual.roompark.domain.contract.AuthContract
import com.biganto.visual.roompark.domain.contract.FlatPlanContract
import io.reactivex.BackpressureStrategy
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 20.04.2020.
 */

class PlanTypesUseCase @Inject constructor(
    private val planContract: FlatPlanContract,
    private val auth:AuthContract
){

    fun fetchNotDownloadedPlansSizes() =
        auth.currentUser()
            .map { user ->
                val list =
                    user.deals?.map { it.estate as EstateEntity }?.toMutableList()
                        ?: mutableListOf()
                list.addAll(user.estates?.map { it as EstateEntity } ?: arrayListOf())
                list
            }
            .flatMapIterable { it }
            .toFlowable(BackpressureStrategy.BUFFER)
            .parallel(4)
            .runOn(Schedulers.io())
            .flatMap{
                planContract.getPlanTypes(it.id)
                    .map { plantypes -> plantypes.flatMap{ plan -> plan.featuresVariants} }
                    .toFlowable(BackpressureStrategy.BUFFER)
            }
            .sequential()
            .toList()
            .toObservable()
            .map { it.flatten() }
            .doOnNext { Timber.d(" plaanzz: $it") }
            .map { list ->
                list.sumBy{ if (planContract.getPlanFile(it).exists()) 0 else 1 }
            }




}


